###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['CustomLayoutEditWdg', 'CustomLayoutEditTestWdg','CustomLayoutHelpWdg', 'CustomLayoutEditSaveCmd']

from pyasm.search import Search, SearchType
from pyasm.web import DivWdg, Table, HtmlElement, SpanWdg, Widget
from pyasm.widget import IconWdg
from pyasm.widget import TextWdg, TextAreaWdg, XmlWdg
from pyasm.command import Command
from pyasm.common import XmlException, Xml,  TacticException
from tactic.ui.common import BaseRefreshWdg

from tactic.ui.widget import ButtonRowWdg, ButtonNewWdg, ActionButtonWdg, SwapDisplayWdg


class CustomLayoutHelpWdg(BaseRefreshWdg):
    '''Showing sample code when clicking on the Show hint button'''
    def get_display(my):
        data = my.kwargs.get('data')

        widget = TextAreaWdg()
        widget.add_styles("max-width: 600px; width: 500px; height: 300px")
        # don't parse it as it doesn't like <%, since we are providing these, should be safe.
        """
        # parse the xml to see if it is valid
        try:
            Xml(string=data, strip_cdata=True)
        except XmlException, e:
            widget.add( IconWdg("XML Parse Error", IconWdg.ERROR) )
            span = SpanWdg()
            span.add_style('color: #f44')
            span.add( "Error parsing xml [%s]:" % e.__str__() )
            widget.add(span)
        """
        value = Xml.to_html(data)
       
        widget.add(value)
        return widget

class CustomLayoutEditWdg(BaseRefreshWdg):



    def get_title_wdg(my, title, content_id=None, is_on=True):
        title_wdg = DivWdg()

        title_area = Widget()
        text  = HtmlElement.b('%s:'%title)
        text.add_style('float: left')
       
        title_area.add(text)
        swap = SwapDisplayWdg(title=title_area, is_on=is_on)
        title_wdg.add(swap)
        if content_id:
            swap.set_content_id(content_id)

        title_wdg.add_gradient("background", "background", -10)
        title_wdg.add_styles("padding: 5px")
        title_wdg.add_border()

        hint = None

        if title == "Behaviors":
            hint = ButtonNewWdg(title="Show hint", icon=IconWdg.HELP)
            data = '''<behavior class="custom_css_class">
                    { "type": "click_up", 
                    "cbjs_action": "spt.alert('clicked')"}
</behavior>''' 
           
            
        elif title == "HTML":
            hint = ButtonNewWdg(title="Show hint", icon=IconWdg.HELP)

            data = '''<div><div><b>Layout Title</b></div><br/>
<%
    x = 7
    y = 28
%>     
<br/>
<div>Select a project</div>
<br/>
</div>
            
            
<element name="user_list" title="Users" width="100%">
  <display class="tactic.ui.panel.FastTableLayoutWdg">
    <search_type>sthpw/login</search_type>
    <view>table</view>
    <do_search>true</do_search>
    <show_select>false</show_select>
    <search_limit>${y}</search_limit>
  </display>
</element>
    '''
        if hint:
            hint.add_behavior({
                'type': 'click_up',
                'data' : data,
                'cbjs_action': '''spt.api.load_popup('Hint for %s', 'tactic.ui.tools.CustomLayoutHelpWdg', {data : bvr.data});'''% title
                })
            text.add_style('margin-top: 6px')
            hint.add_style('float: left')
            hint.add_style('margin-top: -5px')
            title_area.add(hint)


        return title_wdg




    def get_display(my):

        top = my.top
        top.add_class("spt_custom_layout_top")
        top.add_style("padding: 20px")
        top.add_color("color", "color")
        top.add_color("background", "background")
        my.set_as_panel(top)

        inner = DivWdg()
        top.add(inner)

        table = Table()
        inner.add(table)
        table.add_row()

        search = Search("config/widget_config")
        search.add_filter("search_type", "CustomLayoutWdg")
        search.add_order_by("view")
        configs = search.get_sobjects()


        view = my.kwargs.get("view")
        if view == '__new__':
            cur_config = None
        if view:
            search = Search("config/widget_config")
            search.add_filter("search_type", "CustomLayoutWdg")
            search.add_filter("view", view)
            cur_config = search.get_sobject()
        else:
            view = ""
            cur_config = None


        tr, shelf = table.add_row_cell()
        shelf_wdg = DivWdg()
        shelf.add(shelf_wdg)
        shelf_wdg.add_style("height: 30px")

        shelf_wdg.add_style("height: 35px")
        shelf_wdg.add_color("background", "background", -10)
        shelf_wdg.add_style("padding: 3px")
        shelf_wdg.add_style("margin: -2px -2px -1px -2px")
        shelf_wdg.add_border()


        # refresh button
        button_row = ButtonRowWdg()
        shelf_wdg.add(button_row)
        button_row.add_style("float: left")
        button = ButtonNewWdg(title="Refresh", icon=IconWdg.REFRESH)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            spt.app_busy.show("Refreshing ...")
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            spt.panel.refresh(top);
            spt.app_busy.hide();
            '''
        } )


        # add new button
        button = ButtonNewWdg(title="Add New", icon=IconWdg.ADD)
        button_row.add(button)


        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            top.setAttribute("spt_view", "__new__");
            spt.panel.refresh(top);
            '''
        } )


        button = ButtonNewWdg(title="Save", icon=IconWdg.SAVE)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var values = spt.api.Utility.get_input_values(top, null, false);

            var cmd = 'tactic.ui.tools.CustomLayoutEditSaveCmd';
            var html = values.html;
            var style = values.style;
            var view = values.view;
            var behavior = values.behavior;

            if (!view) {
                spt.alert("A view name must be provided to save");
                return;
            }

            spt.app_busy.show("Saving ["+view+"] ...")

            var kwargs = {
                view: view,
                html: html,
                style: style,
                behavior: behavior

            }
            spt.app_busy.show("Saving Custom Layout");
            var server = TacticServerStub.get();
            try {
                server.execute_cmd(cmd, kwargs);
                spt.app_busy.hide();
            } catch(e) {
                spt.app_busy.hide();
                spt.error(spt.exception.handler(e));
                return;
            }
            

            top.setAttribute("spt_view", view);
            spt.panel.refresh(top);

            '''
        } )



        button = ButtonNewWdg(title="Test", icon=IconWdg.ARROW_RIGHT)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var values = spt.api.Utility.get_input_values(top, null, false);

            var class_name = 'tactic.ui.tools.CustomLayoutEditTestWdg';
            spt.panel.load_popup("Test Custom Layout", class_name, values);

        ''' } )


        table.add_row()

        table.add_border()


        left = table.add_cell()
        left.add_color("background", "background3")

        left_div = DivWdg()
        left.add(left_div)
        left.add_style("vertical-align: top")
        left_div.add_style("width: 250px")
        left_div.add_color("color", "color3")
        left_div.add_color("height", "100%")

        left_div.add_style("padding-bottom: 15px")
        left_div.set_unique_id()
        left_div.add_smart_styles( "spt_custom_layout_item", {
            'padding': '3px 6px 3px 10px'
        } )
        left_div.add_relay_behavior( { 
            'type': 'mouseup',
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var content = top.getElement(".spt_custom_layout_content");
            var view = bvr.src_el.getAttribute("spt_view");

            top.setAttribute("spt_view", view)
            spt.panel.refresh(top);
            '''
        } )


        bg_color = left_div.get_color("background3")
        bg_color2 = left_div.get_color("background3", -10)

        left_div.add_relay_behavior( {
            'type': 'mouseover',
            'bg_color': bg_color2,
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            bvr.src_el.setStyle("background-color", bvr.bg_color);
            '''
        } )

        left_div.add_relay_behavior( {
            'type': 'mouseout',
            'bg_color': bg_color2,
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            bvr.src_el.setStyle("background-color", "");
            '''
        } )
        
        error_msgs = []


        title_wdg = DivWdg()
        left_div.add(title_wdg)
        title_wdg.add("<b>Custom Layout Views</b>")
        title_wdg.add_color("color", "color3")
        title_wdg.add_gradient("background", "background3", -5)
        title_wdg.add_style("padding: 5px")
        title_wdg.add_style("margin-bottom: 10px")


        for config in configs:
            config_div = DivWdg()
            left_div.add(config_div)
            config_div.add_class("spt_custom_layout_item")
            config_view = config.get_value("view")
            config_div.add(config_view)
            config_div.add_attr("spt_view", config_view)

            config_div.add_class("hand")


        right = table.add_cell()
        right.add_style("vertical-align: top")
        right_div = DivWdg()
        right.add(right_div)
        right_div.add_color("color", "color")
        right_div.add_color("background", "background", -3)
        right_div.add_style("min-width: 700px")
        right_div.add_style("min-height: 500px")
        right_div.add_style("height: 100%x")
        right_div.add_class("spt_custom_layout_content")

        right_div.add_style("padding: 20px")

        if view != '__new__' and not cur_config:
            nothing_wdg = DivWdg()
            right_div.add(nothing_wdg)
            nothing_wdg.add_border()
            nothing_wdg.add("<b>No Layout Selected</b>")
            nothing_wdg.add_style("padding: 30px")
            nothing_wdg.add_color("color", "color3")
            nothing_wdg.add_color("background", "background3")
            nothing_wdg.add_style("width: 300px")
            nothing_wdg.add_style("text-align: center")
            nothing_wdg.add_style("margin: 60px auto")

        else:

            if view == '__new__':
                html = ''
                style = ''
                behavior_nodes = []
                htmls = []
            else:
                xml = cur_config.get_xml_value("config", strip_cdata=True)

                style_node = xml.get_value("config/%s/html/style" % view)
                style = style_node

                behavior_nodes = xml.get_nodes("config/%s/behavior" % view)

                html_nodes = xml.get_nodes("config/%s/html/*" % view)
                htmls = []
                for node in html_nodes:
                    if xml.get_node_name(node) == 'style':
                        continue
                    # check for user error of putting behavior nodes inside html
                    if xml.get_node_name(node) == 'behavior':
                        bvr_html = xml.to_string(node)
                        error_msgs.append("behavior node found inside html node [%s]. <br><br>It should be on the same level as the html node. Please put them in the Behaviors text box in the Custom Layout Editor.<br> "%bvr_html)
                        #continue
                    htmls.append(xml.to_string(node))

                if not html_nodes:
                    try:
                        raw_data = cur_config.get_value('config')
                        Xml(string=raw_data, strip_cdata=True)
                    except XmlException, e:
                        for text in ['config', view, 'html']:
                            raw_data = raw_data.replace('<%s>' %text, '')
                            raw_data = raw_data.replace('</%s>'%text, '')
                            raw_data = raw_data.replace( "<![CDATA[\n<%", "<%")
                            raw_data = raw_data.replace( "%> ]]>", "%>")
                        htmls.append(raw_data)
                        
                        value = Xml.to_html(raw_data)
                        pre = HtmlElement.pre(value)
                        pre.add_attr("wrap", "true")
                        error_msgs.append("Syntax error found: <br> %s<br><br> %s" %(e.__str__(), pre.get_buffer_display()))

            if error_msgs:
                left_div.add_behavior({
                    'type': 'load',
                    'error_msg' : error_msgs,
                    'cbjs_action' : '''spt.alert(bvr.error_msg.join('<br>'), {type : 'html'});'''
                })

            text = TextWdg("view")
            content_id = text.set_unique_id()

            title_wdg = my.get_title_wdg("View", content_id)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_class("spt_view")
            text.add_style("width: 100%")
            text.add_style("font-size: 12px")
            text.add_style("height: 24px")
            text.add_style("font-family: courier")
            if view != '__new__':
                text.set_value(view)

            right_div.add("<br/>"*2)


            # html
            text = TextAreaWdg("html")
            content_id = text.set_unique_id()

            title_wdg = my.get_title_wdg("HTML", content_id)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 400px")
            text.add_style("min-height: 400px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")
            html = ''.join(htmls)
            if html:
                text.set_value(html)


            right_div.add("<br/>"*2)

            # styles
            text = TextAreaWdg("style")
            content_id = text.set_unique_id()
            text.add_style("display: none")

            title_wdg = my.get_title_wdg("Styles", content_id, is_on=False)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 300px")
            text.add_style("min-height: 300px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")
            text.set_value(style)



            right_div.add("<br/>"*2)

            # behaviors
            text = TextAreaWdg("behavior")
            content_id = text.set_unique_id()
            text.add_style("display: none")

            title_wdg = my.get_title_wdg("Behaviors", content_id, is_on=False)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 300px")
            text.add_style("min-height: 300px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")

            xx = []
            for behavior_node in behavior_nodes:
                xx.append( xml.to_string(behavior_node))
            behavior_str = "\n\n".join(xx)

            text.set_value(behavior_str)







        if my.kwargs.get("is_refresh") == 'true':
            return inner
        else:
            return top


class CustomLayoutEditTestWdg(BaseRefreshWdg):

    def get_display(my):

        html = my.kwargs.get("html")
        style = my.kwargs.get("style")
        view = my.kwargs.get("view")
        behavior = my.kwargs.get("behavior")

        config_xml = CustomLayoutEditSaveCmd.build_xml(view, html, style, behavior)

        top = my.top
        top.add_style("padding: 20px")
        top.add_border()
        top.add_color("color", "color")
        top.add_color("background", "background")

        view = my.kwargs.get("view")


        from tactic.ui.panel import CustomLayoutWdg
        layout = CustomLayoutWdg(config_xml=config_xml, view=view, include_mako=True)
        #layout = CustomLayoutWdg(view=view, include_mako=True)
        top.add(layout)


        return top



class CustomLayoutEditSaveCmd(Command):

    def build_xml(cls, view, html, style=None, behavior=None):

        
        # build up the custom layout
        html = html.strip()
        html = html.replace("<%", "<![CDATA[\n<%")
        html = html.replace("%>", "%> ]]>")

        if style:
            style = style.strip()

        layout = []
        layout.append("<config>")
        layout.append("<%s>" % view)
        layout.append("<html>")


        if style:
            layout.append('''<style type="text/css">''')
            layout.append(style)
            layout.append('''</style>''')
        if html:
            #layout.append('''<div class="spt_top">''')
            layout.append(html)
            #layout.append('''</div>''')




        layout.append("</html>")

        # FIXME: this is very tenuous
        if behavior:
            if behavior.find('<![CDATA[') != -1:
                raise TacticException("CDATA is automatically added when it is saved. Do not include any CDATA tag in behavior.")
            import re
            behavior = behavior.strip()

            p = re.compile("(<behavior.*?>)")
            behavior = p.sub("\\1<![CDATA[", behavior)
            behavior = behavior.replace("</behavior>", "]]></behavior>")

            layout.append(behavior)


        layout.append("</%s>" % view)
        layout.append("</config>")

        config_xml = "\n".join(layout)



        #f = open("/tmp/tt.xml", 'w')
        #f.write(config_xml)
        #f.close()

        #f = open("/tmp/tt.xml")
        #config_xml = f.read()
        #f.close()

        return config_xml

    build_xml = classmethod(build_xml)



    def execute(my):

        html = my.kwargs.get("html")
        style = my.kwargs.get("style")
        view = my.kwargs.get("view")
        behavior = my.kwargs.get("behavior")

        if html and html.find('<![CDATA[') != -1:
            raise TacticException("CDATA is automatically added when it is saved. Do not include any CDATA tag in HTML section.")

        if style and style.find('<![CDATA[') != -1:
            raise TacticException("Do not include any CDATA tag in Styles section.")

        config_xml = my.build_xml(view, html, style, behavior)


        search = Search("config/widget_config")
        search.add_filter("search_type", "CustomLayoutWdg")
        search.add_filter("view", view)
        sobject = search.get_sobject()
        if not sobject:
            sobject = SearchType.create("config/widget_config")
            sobject.set_value("search_type", "CustomLayoutWdg")
            sobject.set_value("view", view)

        sobject.set_value("config", config_xml)
        sobject.commit()






